/*****************************************************************************
		     PCI-HUW  sample program Ver1.2
			    UNI-WIRE SYSTEM
	       Copyright (c) NKE Corporation 2000.04.19
******************************************************************************/

#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>

/*////////////////// define PCI-Bus Information  //////////////////////*/
#define     PciBiosInt               	    0x1a
#define     PciFunctionId               	0xb1
#define     GetPciBiosPresentStatus       	0x01
#define     FindPciDevice                	0x02
#define     ReadPciConfigurationWord      	0x09
#define     ConfigRegDeviceIdOffset       	0x02
#define     ConfigRegSubsystemIdOffset  	0x2e
#define     ConfigRegBaseAddressOffset    	0x10
#define     ConfigRegInterruptLineOffset   	0x3c
#define     MaxBoardNumber                 	16
#define     PLXVenderId                    	0x10B5
#define     PCI9050DeviceId                	0x9050
#define     PCIHUWSubsystemId              	0x1132
#define     PCIHUWIoAddressOffset          	8
#define	    PCIHUWBoardNumberOffset	    	12

/*//////////////// structure for PCI-Bus Information //////////////////*/
struct _pci
   {
	int     DeviceId;
	int     SubsystemId;
	int	BoardNum;
	int	IoStart;
	int     IoAddress[12];
	int	BoardNo;
   };

int     PciInfo( struct _pci * );
int 	getch(void);
int	tolower(int);
void    main( void );

/*///////////////////// get PCI-Bus information ///////////////////////*/
int     PciInfo( struct _pci *TargetDevice )
{
	union   REGS    IRegs;
	union   REGS    ORegs;
	struct  SREGS   SRegs;
	int     NumberOfFoundTargets = 0;
	char    PciBusNumber[MaxBoardNumber];
	char    PciDeviceFunctionNumber[MaxBoardNumber];
	int     i, j, k;

/*/////////////////////// get PCI BIOS status  ////////////////////////*/
	IRegs.h.ah = PciFunctionId;
	IRegs.h.al = GetPciBiosPresentStatus;
	int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
	if ( !ORegs.h.ah && ( ORegs.x.dx == 0x4350 ) ) {

/*///////////////////////// Search PCI device /////////////////////////*/
		for ( j = 0; j < MaxBoardNumber; j++ ) {
			IRegs.h.ah = PciFunctionId;
			IRegs.h.al = FindPciDevice;
			IRegs.x.cx = PCI9050DeviceId;
			IRegs.x.dx = PLXVenderId;
			IRegs.x.si = j;
			int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
			if ( ORegs.h.ah )	break;
			PciBusNumber[NumberOfFoundTargets] = ORegs.h.bh;
			PciDeviceFunctionNumber[NumberOfFoundTargets] = ORegs.h.bl;
			NumberOfFoundTargets++;
		}
		if ( NumberOfFoundTargets ) {
			for ( i = 0; i < NumberOfFoundTargets; i++ ) {

/*////////////////////////// Get Device ID  ///////////////////////////*/
				IRegs.h.ah = PciFunctionId;
				IRegs.h.al = ReadPciConfigurationWord;
				IRegs.h.bh = PciBusNumber[i];
				IRegs.h.bl = PciDeviceFunctionNumber[i];
				IRegs.x.di = ConfigRegDeviceIdOffset;
				int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
				( TargetDevice + i )->DeviceId = ORegs.x.cx;

/*//////////////////////// Get Subsystem ID ///////////////////////////*/
				IRegs.h.ah = PciFunctionId;
				IRegs.h.al = ReadPciConfigurationWord;
				IRegs.h.bh = PciBusNumber[i];
				IRegs.h.bl = PciDeviceFunctionNumber[i];
				IRegs.x.di = ConfigRegSubsystemIdOffset;
				int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
				( TargetDevice + i )->SubsystemId = ORegs.x.cx;

/*///////////////////////// Get I/O address ///////////////////////////*/
				for(k=0; k<12; k++) {
					IRegs.h.ah = PciFunctionId;
					IRegs.h.al = ReadPciConfigurationWord;
					IRegs.h.bh = PciBusNumber[i];
					IRegs.h.bl = PciDeviceFunctionNumber[i];
					IRegs.x.di = ConfigRegBaseAddressOffset+k*2 ;
					int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
					if(ORegs.x.cx & 1)	( TargetDevice + i )->IoAddress[k] = ORegs.x.cx ;
				}

/*//////////////////// Get Board Number address ///////////////////////*/
				IRegs.h.ah = PciFunctionId;
				IRegs.h.al = ReadPciConfigurationWord;
				IRegs.h.bh = PciBusNumber[i];
				IRegs.h.bl = PciDeviceFunctionNumber[i];
				IRegs.x.di = ConfigRegBaseAddressOffset + PCIHUWBoardNumberOffset;
				int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
				if(ORegs.x.cx & 1)	( TargetDevice + i )->BoardNum = (ORegs.x.cx & 0xfffe);
/*////////////////////// Get I/O Start address ////////////////////////*/
				IRegs.h.ah = PciFunctionId;
				IRegs.h.al = ReadPciConfigurationWord;
				IRegs.h.bh = PciBusNumber[i];
				IRegs.h.bl = PciDeviceFunctionNumber[i];
				IRegs.x.di = ConfigRegBaseAddressOffset + PCIHUWIoAddressOffset;
				int86x( PciBiosInt, &IRegs, &ORegs, &SRegs );
				if(ORegs.x.cx & 1)	( TargetDevice + i )->IoStart = (ORegs.x.cx & 0xfffe);
			}
		}
	}
	return NumberOfFoundTargets;
}

/*/////////////////////////////// main ////////////////////////////////*/
void    main( void )
{
	struct _pci	PLX[MaxBoardNumber];
	struct _pci	HUW[16];
	int		NumberOfPLX,NumberOfHUW=0;
	int		HUWBoardNo;
	int		i,j;
	int		n,dt;

	int		ioset = 0x0c;
	int		ioadd = 0x32;
	int		outdata = 0x55;
	int		outadd = 0x08;

	printf("\n\n");
	printf( "       PCI-HUW sample program \n\n" );
	NumberOfPLX = PciInfo( PLX );
	if ( NumberOfPLX ) {
		for ( i = 0; i < NumberOfPLX; i++ ) {
			if(PLX[i].SubsystemId == PCIHUWSubsystemId) {
				HUWBoardNo=(inportb(PLX[i].BoardNum) & 0x000F);
				PLX[i].BoardNo=HUWBoardNo;
				HUW[HUWBoardNo]=PLX[i];
				NumberOfHUW=NumberOfHUW+1;
			}
		}
	}
	if(NumberOfHUW ) {
		printf("There is(are) %d PCI-HUW \n",NumberOfHUW );
		for( j = 0; j < 16; j++ ) {
		    if(HUW[j].SubsystemId == PCIHUWSubsystemId)   printf( "   Board No [%01x]   I/O Start [%4xh] \n", HUW[j].BoardNo,HUW[j].IoStart );
		}
		printf("\n");
		printf("input U/W address 0-7 \n");
		printf("output U/W address 64-71  data %02x \n",outdata);
		printf(" select Board No ");
		n = toupper(getch())-'0';
		if( n >= 0x10 )	dt=n-0x07;
		else        		dt=n;
		outportb((HUW[dt].IoStart+ioadd),ioset);
		printf("[%1x]\n", dt );
		printf("input U/W address 0-7  data [%02x] \n",inportb(HUW[dt].IoStart));
		outportb((HUW[dt].IoStart+outadd),outdata);
		printf("output U/W address 64-71  data %02x  \n",outdata);
	}
	else    printf( "Cannot find PCI-HUW \n" );
}
